# Imports.

import numpy as np

from . import Preprocessing
from . import Utilities as util
from . import Colour_plotting
from .. import Global_Utilities as gu

# Main function definition.

def Colour_Analysis_Main( ip ):

    print( "Colour analysis has begun." )

    file_data, data = [], []

    if ip.read_files:

        file_data, data = Preprocessing.read_files_and_preprocess( ip.directory, ip.data_directory, ip.merge_groups )

    if ip.write_csv:

        Preprocessing.write_csv( ip.output_directory, file_data, data )

    if ip.read_csv:

        file_data, data = Preprocessing.read_csv( ip.directory, ip.output_directory, ip.merge_groups )

    # Data of form [L, a, b]. Data of type lists of numpys.

    gu.print_files_read( len( data[0] ), 120 )

    if ip.remove_files:

        Preprocessing.remove_files( file_data, data, ip.remove_files_string )

    if ip.compute_mean:

        Preprocessing.compute_mean( ip.output_directory, file_data, data )

    if ip.read_mean:

        Preprocessing.read_mean( ip.output_directory, data )

    # Data now of form [L, a, b, mL, ma, mb]. Data of type lists of numpys.

    if ip.extract_features:

        util.extract_colour_features( ip.output_directory, file_data, data )

    if ip.read_and_analyse_features:

        util.read_and_analyse_features( ip, file_data )

    if ip.plot_data:

        Colour_plotting.plot_data( ip, file_data, data )

    if ip.sandbox:

        util.sandbox( ip.directory, ip.output_directory, file_data, data )

    print( "Colour analysis complete." )



# Imports.

import os
import glob
import shutil
import re
import numpy as np
import pandas as pd

from .. import Global_Utilities as gu

# Function definitions.

def read_raw_data_file_1( filename, resin_data, file_data, data ):

    sheet = "Sheet 1"

    df = pd.read_excel( filename, sheet )

    resins = df.iloc[:, 0].tolist()

    for r in resins:

        file_data.append( [int( r ), 0, resin_data.loc[int( r )]["Label"] + ".{}".format( 0 ), ""] )

    for i in range( 1, len( df.columns ) ):

        for j in range( len( df ) ):

            data[i - 1].append( df.iloc[j, i] )

def extract_raw_data( directory, data_directory ):
    '''Extract the raw data from the files.'''

    resin_data = gu.get_list_of_resins_data( directory ) # Obtain the spreadsheet of data for the resins.

    file_data, data = [], [[] for i in range( 14 )]

    read_raw_data_file_1( data_directory + "Raw Data.xlsx", resin_data, file_data, data )

    return file_data, data

def standardise_data( data ):
    '''Standardise data.'''

    pass

def add_description_to_file_data( file_data ):
    '''Add descriptions in the form of letters to each specimen.'''

    pass

def read_files_and_preprocess( directory, data_directory, merge_groups ):
    '''Read files and preprocess data.'''

    file_data, data = extract_raw_data( directory, data_directory )

    standardise_data( data )

    add_description_to_file_data( file_data )

    if merge_groups:

        gu.merge( file_data )

    return file_data, data

def write_csv( output_directory, file_data, data, name_appendage = "" ):
    '''Write read and preprocessed data to a .csv file.'''

    pass

    # array = data[0][:, np.newaxis]
    #
    # for i in range( len( data[1] ) ):
    #
    #     array = np.hstack( (array, data[1][i][:, np.newaxis]) )
    #
    # np.savetxt( output_directory + "ESCR/Condensed_Data/escr_data" + name_appendage + ".csv", array, delimiter = ",", fmt = "%.2f" )
    #
    # array = np.array( file_data )
    #
    # np.savetxt( output_directory + "ESCR/Condensed_Data/file_data" + name_appendage + ".csv", array, delimiter = ",", fmt = "%s" )

def read_csv( directory, output_directory, merge_groups, name_appendage = "" ):
    '''Read the preprocessed .csv file.'''

    pass

    # resin_data = gu.get_list_of_resins_data( directory, name_appendage ) # Obtain the spreadsheet of data for the resins.
    #
    # file_data = []
    #
    # df = pd.read_csv( output_directory + "ESCR/Condensed_Data/file_data" + name_appendage + ".csv", sep = ",", header = None )
    #
    # for i in range( len( df.index ) ):
    #
    #     resin = int( df.iloc[i, 0] )
    #     specimen = int( df.iloc[i, 1] )
    #
    #     file_data.append( [resin, specimen, resin_data.loc[resin]["Label"] + ".{}".format( specimen ), ""] )
    #
    # data = []
    #
    # df = pd.read_csv( output_directory + "ESCR/Condensed_Data/escr_data" + name_appendage + ".csv", sep = ",", header = None )
    #
    # data.append( df.iloc[:, 0].to_numpy( dtype = np.float64 ) )
    #
    # percentages = []
    #
    # for i in range( 1, len( df.columns ) ):
    #
    #     percentages.append( df.iloc[:, i].to_numpy( dtype = np.float64 ) )
    #
    # data.append( percentages )
    #
    # data.append( np.array( [24, 48, 72, 96] ) )
    #
    # add_description_to_file_data( file_data )
    #
    # if merge_groups:
    #
    #     gu.merge( file_data )
    #
    # return file_data, data

def remove_files( file_data, data, descriptors_to_remove = "" ):
    '''Remove files not needed/wanted for analysis by searching for letters in file descriptions.'''

    return file_data, data

    # files_to_remove = []
    #
    # for i in range( len( file_data ) ):
    #
    #     s = file_data[i][3]
    #
    #     for l in descriptors_to_remove:
    #
    #         if s.find( l ) > -0.5:
    #
    #             files_to_remove.append( i )
    #
    #             break
    #
    # files_to_remove.reverse()
    #
    # for r in files_to_remove:
    #
    #     file_data.pop( r )
    #     data[1].pop( r )
    #
    # return file_data, data

def compute_mean( output_directory, file_data, data, name_appendage = "" ):
    '''Compute the mean data for each resin.'''

    pass

def read_mean( output_directory, data, name_appendage = "" ):
    '''Read the computed means for each resin from a file.'''

    pass

def copy_data( output_directory, name_appendage_1, name_appendage_2 ):
    '''Delete data.'''

    pass

    # shutil.copyfile( output_directory + "ESCR/Condensed_Data/escr_data" + name_appendage_1 + ".csv", output_directory + "ESCR/Condensed_Data/escr_data" + name_appendage_2 + ".csv" )
    #
    # shutil.copyfile( output_directory + "ESCR/Condensed_Data/file_data" + name_appendage_1 + ".csv", output_directory + "ESCR/Condensed_Data/file_data" + name_appendage_2 + ".csv" )

def delete_data( output_directory, name_appendage ):
    '''Delete data.'''

    pass

    # pattern = re.compile( name_appendage + "\\.csv$" )
    #
    # for path in glob.glob( output_directory + "ESCR/Condensed_Data/*" ):
    #
    #     if pattern.search( path ):
    #
    #         os.remove( path )
    #
    # for path in glob.glob( output_directory + "ESCR/Features/*" ):
    #
    #     if pattern.search( path ):
    #
    #         os.remove( path )



# Imports

import pandas as pd

import rHDPE_Data_Analysis.FTIR_Analysis as fa
import rHDPE_Data_Analysis.DSC_Analysis as da
import rHDPE_Data_Analysis.TGA_Analysis as ta
import rHDPE_Data_Analysis.Rheology_Analysis as ra
import rHDPE_Data_Analysis.Colour_Analysis as ca
import rHDPE_Data_Analysis.TT_Analysis as tt
import rHDPE_Data_Analysis.SHM_Analysis as sa
import rHDPE_Data_Analysis.TLS_Analysis as tls
import rHDPE_Data_Analysis.ESCR_Analysis as ea

from . import Utilities as util

from .. import Global_Utilities as gu

# Function definitions.

def rerun_compute_features( name ):

    # Parameters.

    ip = fa.Input_Parameters_Class.Input_Parameters()

    fa.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/FTIR_" + name + "_Parameters.numbers", ip )

    # Main function call.

    fa.Analysis.FTIR_Analysis_Main( ip )

    # Parameters.

    ip = da.Input_Parameters_Class.Input_Parameters()

    da.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/DSC_" + name + "_Parameters.numbers", ip )

    # Main function call.

    da.Analysis.DSC_Analysis_Main( ip )

    # Parameters.

    ip = ta.Input_Parameters_Class.Input_Parameters()

    ta.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/TGA_" + name + "_Parameters.numbers", ip )

    # Main function call.

    ta.Analysis.TGA_Analysis_Main( ip )

    # Parameters.

    ip = ra.Input_Parameters_Class.Input_Parameters()

    ra.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/Rheology_" + name + "_Parameters.numbers", ip )

    # Main function call.

    ra.Analysis.Rheology_Analysis_Main( ip )

    # Parameters.

    ip = ca.Input_Parameters_Class.Input_Parameters()

    ca.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/Colour_" + name + "_Parameters.numbers", ip )

    # Main function call.

    ca.Analysis.Colour_Analysis_Main( ip )

    # Parameters.

    ip = tt.Input_Parameters_Class.Input_Parameters()

    tt.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/TT_" + name + "_Parameters.numbers", ip )

    # Main function call.

    tt.Analysis.TT_Analysis_Main( ip )

    # Parameters.

    ip = sa.Input_Parameters_Class.Input_Parameters()

    sa.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/SHM_" + name + "_Parameters.numbers", ip )

    # Main function call.

    sa.Analysis.SHM_Analysis_Main( ip )

    # Parameters.

    ip = tls.Input_Parameters_Class.Input_Parameters()

    tls.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/TLS_" + name + "_Parameters.numbers", ip )

    # Main function call.

    tls.Analysis.TLS_Analysis_Main( ip )

    # Parameters.

    ip = ea.Input_Parameters_Class.Input_Parameters()

    ea.Input_Parameters_Class.read_parameters_from_numbers_file( ip.directory + "Input_Parameters/ESCR_" + name + "_Parameters.numbers", ip )

    # Main function call.

    ea.Analysis.ESCR_Analysis_Main( ip )

def read_files_and_preprocess( ip, datasets_to_read = [], normalised = True, minimal_datasets = False, feature_1 = "", feature_2 = "", name_appendage = "" ):

    authorised = False

    if type( datasets_to_read ) == int:

        datasets_to_read = [datasets_to_read]

    ip.datasets_to_read = datasets_to_read

    dataset_names = ["FTIR", "DSC", "TGA", "Rheology", "TT", "Colour", "SHM", "TLS", "ESCR", "FTIR2", "FTIR3", "TGA_SB", "SAXS", "GPC", "SAXS"]
    dataset_directories = ["FTIR/Features/", "DSC/Features/", "TGA/Features/", "Rheology/Features/", "TT/Features/", "Colour/Features/", "SHM/Features/", "TLS/Features/", "ESCR/Features/", "FTIR/Integral_Analysis/", "FTIR/Component_Analysis/Features/", "TGA/Sandbox/", "SAXS/Features/", "GPC/Features", "Global/TM/"]

    normalised_ext = ""

    dataset, std_dataset = [], []

    for i in range( len( dataset_names ) ):

        if datasets_to_read:

            if (i + 1) not in datasets_to_read:

                continue

        n = dataset_names[i]

        if not normalised:

            if n == "FTIR2" or n == "FTIR3" or n == "TGA_SB":

                normalised_ext = ""

            else:

                normalised_ext = "_Unnormalised"

        df, authorised = gu.read_csv_pipeline( ip, dataset_directories[i], "Mean_Features" + normalised_ext + name_appendage + ".csv", authorised )
        df_std, authorised = gu.read_csv_pipeline( ip, dataset_directories[i], "Std_of_Features" + normalised_ext + name_appendage + ".csv", authorised )

        if minimal_datasets:

            if feature_1 not in df.columns.tolist():

                if feature_2 not in df.columns.tolist():

                    continue

        dataset.append( df )
        std_dataset.append( df_std )

    if ip.sample_mask == None:

        ip.sample_mask = []

    sample_mask = ip.sample_mask.copy()

    if sample_mask == []:

        sample_mask = list( map( int, dataset[0].iloc[:, 0].tolist() ) )

        for i in range( 1, len( dataset ) ):

            samples_present = dataset[i].iloc[:, 0].tolist()
            sample_mask = gu.remove_redundant_samples( sample_mask, samples_present )

    else:

        for i in range( len( dataset ) ):

            samples_present = dataset[i].iloc[:, 0].tolist()
            sample_mask = gu.remove_redundant_samples( sample_mask, samples_present )

    features, feature_names = util.produce_full_dataset_of_features( dataset, sample_mask )

    rank_features = util.rank_features( features )

    features_df = gu.array_with_column_titles_and_label_titles_to_df( features, feature_names, sample_mask )

    rank_features_df = gu.array_with_column_titles_and_label_titles_to_df( rank_features, feature_names, sample_mask )

    std_of_features, _ = util.produce_full_dataset_of_features( std_dataset, sample_mask )

    std_of_features_df = gu.array_with_column_titles_and_label_titles_to_df( std_of_features, feature_names, sample_mask )

    #===============

    # Extracting the whole dataset as a .csv.

    # resin_data = gu.get_list_of_resins_data( ip.directory )
    #
    # sample_mask_2 = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23]
    #
    # features_2, feature_names_2, sample_mask_2 = util.compile_full_dataset_of_features( dataset, sample_mask_2 )
    # std_of_features_2, std_of_feature_names_2, sample_mask_2 = util.compile_full_dataset_of_features( std_dataset, sample_mask_2 )
    #
    # features_2_df = gu.array_with_column_titles_and_label_titles_to_df( features_2, feature_names_2, sample_mask_2 )
    # std_of_features_2_df = gu.array_with_column_titles_and_label_titles_to_df( std_of_features_2, std_of_feature_names_2, sample_mask_2 )
    #
    # features_2_df = features_2_df.set_axis( [resin_data.loc[i]["Label"] for i in features_2_df.index], axis = "index" )
    # std_of_features_2_df = std_of_features_2_df.set_axis( [resin_data.loc[i]["Label"] for i in std_of_features_2_df.index], axis = "index" )
    #
    # features_2_df.astype( float ).to_csv( ip.output_directory + "Global/Dataset/Full_Dataset.csv", float_format = "%.5f" )
    # std_of_features_2_df.astype( float ).to_csv( ip.output_directory + "Global/Dataset/Full_Dataset_Std.csv", float_format = "%.5f" )

    #===============

    return features_df, std_of_features_df, rank_features_df



# Imports.

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib as mpl

from .. import Global_Utilities as gu

# Function definitions.

def plot_data( ip, file_data, data, savefig = True, name_appendage = "" ):

    resin_data = gu.get_list_of_resins_data( ip.directory, name_appendage )

    feature_names, features = gu.csv_to_df_to_array_and_column_titles( ip.output_directory + "SAXS/Features/Features.csv" )

    sample, sample_array, samples_present, samples_present_array = gu.sample_data_from_file_data( file_data )

    samples_to_plot = samples_present
    samples_to_plot = [16, 17, 19, 12, 15, 1, 2, 3, 5, 6, 7, 8, 9, 4, 10, 13, 11, 18, 20, 21, 22, 23]

    specimens = False
    all_specimens = True
    specimen_mask = []

    mean = True

    if mean:

        mean_features = gu.extract_mean_features( features, sample_array, samples_to_plot )

        std_of_features = gu.extract_std_of_features( features, sample_array, samples_to_plot )

        bar_weights = {"Lc": mean_features[:, 3], "La": mean_features[:, 4]}
        hatch = {"Lc": "/", "La": "*"}

        y_pos = np.arange( len( mean_features[:, 0] ) )

        colours = gu.list_of_colours()
        colours = [colours[i] for i in samples_to_plot]

        fig, ax = plt.subplots()
        bottom = np.zeros( len( mean_features[:, 3] ) )

        for label, weight in bar_weights.items():

            ax.bar( y_pos, weight, label = label, bottom = bottom, align = 'center', alpha = 0.5, color = colours, edgecolor = "black", hatch = hatch[label] )

            bottom += weight

        # plt.bar( y_pos, mean_features[:, 0], align = 'center', alpha = 0.5, color = colours )

        plt.xticks( y_pos, [resin_data.loc[i]["Label"] for i in samples_to_plot], rotation = 90 )

        legend_1 = mpatches.Patch( facecolor = "white", edgecolor = "black", hatch = "/",label = "lc" )
        legend_2 = mpatches.Patch( facecolor = "white", edgecolor = "black", hatch = "*",label = "la" )
        plt.legend( handles = [legend_1, legend_2] )

        # plt.xlabel( xlabel )
        plt.ylabel( "d" )
        # plt.title( title )
        #
        # plt.ylim( [55, 75] )
        #
        plt.tight_layout()
        #
        if savefig:

            plt.savefig( ip.output_directory + "SAXS/Plots/Plot.pdf" )

        else:

            plt.show()

        plt.close()





##############
# How I previously obtained a directory.

from pathlib import Path

directory = Path( __file__ ).absolute().parents[1].as_posix() + "/"

##############
# Adding a path to the sys list.

import sys
from pathlib import Path

sys.path.append( Path( __file__ ).absolute().parents[3].as_posix() )

##############

import numpy as np
import matplotlib.pyplot as plt
from sklearn.cluster import AgglomerativeClustering

def plot_dendrogram( model, **kwargs ):

    # Create linkage matrix and then plot the dendrogram

    counts = np.zeros( model.children_.shape[0] )
    n_samples = len( model.labels_ )

    for i, merge in enumerate( model.children_ ):

        current_count = 0

        for child_idx in merge:

            if child_idx < n_samples:

                current_count += 1  # leaf node

            else:

                current_count += counts[child_idx - n_samples]

        counts[i] = current_count

    linkage_matrix = np.column_stack( [model.children_, model.distances_, counts] ).astype( float )

    # Plot the corresponding dendrogram.

    dendrogram( linkage_matrix, **kwargs )

def plot_dendrogram_2( output_directory, features, file_data ):
    """7/9/23: A second way of plotting a dendrogram. Starting from features instead of a distance matrix,
    this works in the same way as the maths of gu.plot_dendrogram. It has the flexibility of defining the metric from which
    the distance matrix is created, but you can't use linkage 'ward' with any metric that isn't 'Euclidean'."""

    model = AgglomerativeClustering( distance_threshold = 0, n_clusters = None, affinity = 'manhattan', linkage = 'single' )

    model = model.fit( features )

    plt.figure( figsize = ( 18, 14 ) )

    labels = np.array( [f[2] for f in file_data] )

    plot_dendrogram( model, labels = labels, count_sort = True )

    ax = plt.gca()
    ax.tick_params( axis = 'x', which = 'major', labelsize = 15 )
    plt.tight_layout()

    plt.savefig( output_directory + "Features/Dendrogram.pdf" )

# The below is copied from Global_Analysis_Old.py

import os
import glob
import pandas as pd
import matplotlib.pyplot as plt
from scipy.cluster.hierarchy import linkage
from scipy.cluster.hierarchy import dendrogram
from scipy.spatial.distance import squareform
import matplotlib.cm as cm
import numpy as np

def plot_distances( output_directory, subdirectory, df, sample_mask, labels = True ):

    fig, ax = plt.subplots()
    im = ax.imshow( df, cmap = cm.coolwarm )
    plt.xticks( np.arange( 0, len( sample_mask ), 1 ) )
    plt.yticks( np.arange( 0, len( sample_mask ), 1 ) )

    ax.set_xticklabels( sample_mask, rotation = 270, fontsize = 6 )
    ax.set_yticklabels( sample_mask, fontsize = 6 )

    for i in range( len( sample_mask ) ):

        for j in range( len( sample_mask ) ):

            ax.text( j, i, "{:.1f}".format( df.iloc[i, j] ), size = 6, ha = "center", va = "center", color = "w" )

    plt.tight_layout()
    ax.invert_yaxis()
    plt.savefig( subdirectory + "/Distances.pdf" )
    plt.close()

dir = "Users/philsmith/Documents/Postdoc/rHDPE_Data_Analysis/Distance_Matrices/"

sample_mask = [11, 14, 10, 4, 13, 21, 23, 18, 22, 20, 2, 3, 17, 16, 19, 1, 15, 12, 6, 5, 7, 9, 8]

os.chdir( dir )

matrices_files = glob.glob( "*" )

matrices = []

for f in matrices_files:

    if f == "Output":

        continue

    df = pd.read_csv( f, sep = "," )

    index_names = df.iloc[:, 0].tolist()

    df.rename( index = {df.index[i]:index_names[i] for i in range( len( index_names ) )}, inplace = True )

    df.drop( columns = df.columns[0], inplace = True )

    matrices.append( df )

for i in range( len( matrices ) ):

    max_val = matrices[i].max().max()
    matrices[i] = matrices[i].divide( max_val )

global_df = matrices[0] + matrices[1]

plot_distances( dir, "Output", global_df, sample_mask )

global_array = global_df.to_numpy()

fig, ax = plt.subplots()

condensed_distance_matrix = squareform( global_array )
linkage_matrix = linkage( condensed_distance_matrix, "single" )

dendrogram( linkage_matrix, labels = sample_mask, color_threshold = 0.65 * max( linkage_matrix[:,2] ), count_sort = True )

plt.xlabel( "PCR Sample" )
plt.title( "Clustering wrt FTIR" )
plt.savefig( "Output/Clustering.pdf" )
plt.close()

######
